% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [result] = ExecCla(labels, Ds, div_names, cvo)

M = size(Ds, 1);
n_divs = size(Ds, 3);

ys = nan(M, n_divs);
for ind = 1:n_divs
    D = symmetrize(Ds(:,:,ind));
    div_name = div_names{ind};

    yy = zeros(M, 1);
    for vnd = 1:cvo.NumTestSets
        if strcmp(class(cvo), 'cvpartition');
            trI = cvo.training(vnd);
            teI = cvo.test(vnd);
        else
            trI = cvo.training{vnd};
            teI = cvo.test{vnd};
        end
        
        % CV
        D_cv = D(trI,trI);
        cvo_cv = cvpartition(size(D_cv,1), 'kfold', 3);
        [C width] = TuneSVM(D_cv, div_name, labels(trI), cvo_cv);
        
        % different divs
        if isempty(strfind(lower(div_name),'ppk')) &&...
                isempty(strfind(lower(div_name),'bc')) &&...
                isempty(strfind(lower(div_name),'pmk'))
            KM = exp(-0.5*(D.^2)./(width.^2)); % gaussian kernel
        else
            KM = D; % inner product
        end
        KM = ProjectPSD(KM, false);
        
        yy(teI) = SVMClaKM(KM, trI, teI, labels, C);
    end
    
    ys(:,ind) = yy;
end

for ind = 1:n_divs
    acc_svm = mean(ys(:,ind) == labels);
    eval(sprintf('acc_svm_%s = acc_svm;', div_names{ind}));
end

clear acc_svm
result = ToSavedWorkspace;

PrintStruct(result, 'acc_*');
