% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [features pca] = ImgMakeBoF(features_raw, imgs, loc, opt)

dop = opt.data_opt;
fop = opt.feat_opt;

%% handling blanks
disp('Handling blanks...');

M = length(features_raw);
features = cell(M, 1);
switch lower(dop.feat_type)
    case 'patch' % patch
        blank_thresh = 2e-2;
        for ind = 1:M
            feat = features_raw{ind};
            [n_f dim] = size(feat);

            % truncate outlier pixels within each patch
            sorted_f = sort(feat, 2, 'ascend');
            lb_ub = sorted_f(:, round(n_f*[0.05, 0.95]));
            for jnd = 1:size(feat,1)
                feat(jnd, feat(jnd,:) < lb_ub(jnd,1)) = lb_ub(jnd,1);
                feat(jnd, feat(jnd,:) > lb_ub(jnd,2)) = lb_ub(jnd,2);
            end

            blank_idx = std(feat, [], 2) < blank_thresh;
            feat(~blank_idx, :) = Normalize(feat(~blank_idx, :), '01', 2);
            switch lower(fop.blank_handler)
                case 'drop'
                    feat = feat(~blank_idx, :);
                case 'fill'
                    mag = 3*(2*sqrt(3)*blank_thresh);
                    feat(blank_idx, :) = mag*rand(sum(blank_idx), dim);
                case 'none'
                    feat(blank_idx, :) = 0;
                otherwise
                    error('unknown blank handler');
            end

            features{ind} = feat;
            TraceIter(ind, 1, M);
        end
    case {'phow', 'sift'}
        blank_thresh = 1000;
        for ind = 1:M
            feat = features_raw{ind};
            [n_f dim] = size(feat);
            
            blank_idx = sum(feat, 2) < blank_thresh;
            switch lower(fop.blank_handler)
                case 'drop'
                    feat = feat(~blank_idx, :);
                case 'fill'
                    mag = 3*(2*blank_thresh/size(feat,2));
                    feat(blank_idx, :) = mag*rand(sum(blank_idx), dim);
                case 'none'
                    feat(blank_idx, :) = 0;
                otherwise
                    error('unknown blank handler');
            end

            features{ind} = feat/255;
            TraceIter(ind, 1, M);
        end
    otherwise
        error('unknown feature type');
end
clear features_raw

%% pca

if fop.pca
    disp('PCA...');
    pca = PCA(CatMat(features,[],1), fop.pca_dim);
    disp(pca);
    features = map(@(f) pca.Project(f), features);
else
    pca = [];
end

%% spatial info

if ~isempty(fop.spatial_info)
    disp('Adding spatial info');
    for m = 1:M
        [h w] = size(imgs{m});
        spa = [];
        if strfind(lower(fop.spatial_info), 'x')
            spa = [spa loc{m}(:,1)/w];
        end
        if strfind(lower(fop.spatial_info), 'y')
            spa = [spa loc{m}(:,2)/h];
        end
        features{m} = [features{m} spa];
        TraceIter(m, 1, M);
    end
end

%% normalize

if ~strcmpi(fop.normalizer, 'none')
    disp('Normalizing features...');
    [dum shifters scalers] = Normalize(CatMat(features,[],1), fop.normalizer, 1);
    features = map(@(f) Normalize(f,{shifters,scalers},1), features);
end
features = map(@(f) single(f), features);
