% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [X_bow X_plsa codebook codes] = ImgMakeBoW(features_raw, imgs, locs, opt)

fop = opt.feat_opt;
bop = opt.bow_opt;

M = length(features_raw);
    
%% spatial info
if ~isempty(fop.spatial_info)
    disp('Adding spatial info into BoW');
    
    for m = 1:M
        [h w] = size(imgs{m});
        spa = [];
        if strfind(lower(fop.spatial_info), 'x')
            spa = [spa locs{m}(:,1)/w];
        end
        if strfind(lower(fop.spatial_info), 'y')
            spa = [spa locs{m}(:,2)/h];
        end
        features_raw{m} = [features_raw{m} spa];
        TraceIter(m, 1, M);
    end
end

%% discretize

[X_raw group_id] = MakeGroupData(features_raw);
[nx dim] = size(X_raw);
X_raw = Normalize(double(X_raw), 'm0v1', 1);
fprintf('Converting %dx%d features into %d words\n', ...
    nx, dim, bop.n_code);

disp('kmeans...');
codebook = vl_kmeans(X_raw(RI(min(1e5, nx), nx), :)', bop.n_code, 'Initialization', 'PLUSPLUS','algorithm', 'ELKAN');
codebook = codebook';

disp('discretizing...');
codes = knnsearch(codebook, X_raw);

X_bow = accumarray([group_id(:), codes(:)], 1);

%% plsa

disp('plsa...');
[Pw_z,Pd_z,Pz,Pz_d] = pLSA_EM(X_bow',bop.n_topic);
X_plsa = Pz_d';
