% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [ K ] = ProjectPSD(K, diag_one)
% [ K ] = ProjectPSD(K, diag_one)
% make K a valid kernel matrix i.e. make it PSD
% diag_one: if the diagonal of K should be forced to one
% Ref: Nicholas J. Higham, Computing the nearest correlation matrix - a
% problem from finance, J. of Numerical Analysis

if nargin < 2
    diag_one = false;
end
assert(~any(isnan(K(:)) & ~any(isinf(K(:)))), ...
       'invalid values found');

lb = 1e-5;
max_iter = 20;
parfor ind = 1:size(K,3)
    k = symmetrize(K(:,:,ind));
   
    if diag_one
        S = 0;
        X = k;
        for iter = 1:max_iter
            XO = X;
            
            R = k - S;
            [V D] = eig(R);
            X = symmetrize(bsxfun(@times,V,max(diag(D),lb)')*V');
            S = X - R;
            k = SetDiag(X, 1);
            
            change = max(abs(XO(:) - X(:)))/max(abs(X(:)));
            if iter > 1 && change < 1e-7
                break;
            end
        end
        
        % check
        d = eigs(k, 1, 'SA');
        if d < lb
            fprintf('KM %d: Min Eig = %g\n', ind, d);
            assert(d < -0.5, 'error too large');
        end
    else
        if eigs(k, 1, 'SA') < 0
            [V D] = eig(k); D = diag(D);
            if min(D) < lb
                fprintf('KM %d: Min eigvalue = %g, truncating to %g\n',...
                    ind,min(D),lb);
                k = bsxfun(@times,V, max(D,lb)')*V';
            end
            idx = argmax(max(abs(diag(k) - 1)));
            if abs(k(idx,idx) - 1) > 1e-2
                fprintf('KM %d: wildest diagonal = %g\n', ind, k(idx,idx));
            end
        end
    end
    
    K(:,:,ind) = symmetrize(k);
end
