% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [C, width, svm_result] = TuneSVM(D, div_name, labels, cvo, options)

if nargin < 5;    options = [];    end
[widths Cs] = GetOptions(options, ...
    'kernel_widths', 2.^(-4:2:10), 'svm_Cs', 2.^(-9:3:15));
assert(length(unique(labels)) == max(labels), 'labels should be 1,...,n_class');
assert(size(D,1) == length(labels), 'inconsistent data');

M = length(labels);
fprintf('Tuning parameters. #Class=%d, #Sample=%d, #Folds=%d\n', ...
    max(labels), M, cvo.NumTestSets);

D = symmetrize(double(D));

if isempty(strfind(lower(div_name),'ppk')) &&...
        isempty(strfind(lower(div_name),'bc')) &&...
        isempty(strfind(lower(div_name),'pmk'))
    width_base = median(D(D>0));
    widths = widths*width_base;
    if isnan(width_base)
        warning('All 0 distance matrix found. Using unit kernel width.');
        widths = 1; 
    end
else
    widths = nan;
end

%% tune width and C

disp('Tuning SVM...');
ys = zeros(M, length(Cs), length(widths));
for wnd = 1:length(widths)
    width = widths(wnd);
    
    fprintf('Making KM %d\n', wnd');
    if ~isnan(width) % gaussian kernel
        KM = exp(-0.5.*(D.^2)./(width^2));
    else % inner product
        KM = D;
    end
    KM = ProjectPSD(KM, false);

    tmp = zeros(M, length(Cs));
    for cnd = 1:length(Cs)
        C = Cs(cnd);
        fprintf('Training SVM /w KM%d, C=%g\n', wnd, C);

        for vnd = 1:cvo.NumTestSets
            trI = cvo.training(vnd);
            teI = cvo.test(vnd);

            tmp(teI,cnd) = SVMClaKM(KM, trI, teI, labels, C, false);
        end
    end
    ys(:,:,wnd) = tmp;
end
accs_svm = squeeze(mean(ys == repmat(labels(:), [1, length(Cs), length(widths)]), 1));
if isnan(widths); 
    accs_svm = accs_svm'; 
end
idx = argmax(accs_svm(:));
[ci, wi] = ind2sub(size(accs_svm), idx);
C = Cs(ci);
width = widths(wi);
svm_result = struct('Cs',Cs,'widths',widths,'accs',accs_svm);
