% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [D] = L2Dist(A, B, sqr, col_wise)
% [D] = L2Dist(A, B, sqr, col_wise)
%    A - (MxD) matrix or (DxM) if col_wise (default false)
%    B - (NxD) matrix or (DxN) if col_wise. if [], then equal to A
%    D - (MxN) Euclidean distances between vectors in A and B
%    D = ||A-B|| = sqrt ( ||A||^2 + ||B||^2 - 2*A.B )
% if not sqr (default true): 1) the result is the sqruared distance. 
%             2) the distance might be negative

if nargin < 4
    col_wise = false;
    if nargin < 3
        sqr = true;
        if nargin < 2
            B = [];
        end
    end
end

if ~isreal(A) || ~isreal(B)
   warning('ex:EncDist2', 'cannot handle complex numbers'); 
end

if col_wise
    if isempty(B)
        AA = sum(A.^2, 1); 
        D = bsxfun(@plus, bsxfun(@plus, -2*(A'*A), AA'), AA);
    else
        D = bsxfun(@plus, bsxfun(@plus, -2*(A'*B), ...
            sum(A.^2, 1)'), sum(B.^2, 1));
    end
else
    if isempty(B)
        AA = sum(A.^2, 2);
        D = bsxfun(@plus, bsxfun(@plus, -2*(A*A'), AA), AA');
    else
        D = bsxfun(@plus, bsxfun(@plus, -2*(A*B'), ...
            sum(A.^2, 2)), sum(B.^2, 2)');
    end
end

if nargin < 3 || sqr
    D = sqrt(max(0, D));
end
