% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [ X group_id data_idx] = MakeGroupData( data, groups )
%[ X group_id] = MakeGroupData( data, groups )
% make group data out of a cell array or matrix group idx
% if data is cell array, the each cell is a group, one row per sample
% if data is matrix, then cell array 'groups' gives the membership. data_idx is the index of X in data.

if iscell(data)
    data = data(:);
    N = cellfun(@(d) size(d, 1), data);
    cN = [0; cumsum(N)];
    X = CatMat(data, [], 1);
    data_idx = [];
else
    N = cellfun(@length, groups);
    cN = [0; cumsum(N)];
    data_idx = CatVec(groups);
    X = data(data_idx, :);
end

group_id = int32(zeros(cN(end), 1));
for ind = 1:length(N)
    group_id(cN(ind) + 1:cN(ind + 1)) = ind;
end
