% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [imgs file_names] = ReadImages(file_filter, options)
%[imgs file_names] = ReadImages(file_filter, options)
% read images with the file name filter

if nargin < 2; options = []; end
[sz, max_num, sampler] = GetOptions(options,...
    'sz',[],'max_num',inf,'sampler','uniform');

file_names = ListFiles([file_filter, '/*.jpg']);
file_names = [file_names, ListFiles([file_filter, '/*.png'])];
file_names = [file_names, ListFiles([file_filter, '/*.bmp'])];
if isempty(file_names)
    error('no image to read');
end

n = min(max_num, length(file_names));
if n < length(file_names) % if subsampling is needed
    switch lower(sampler)
        case 'first'
            idx = 1:n;
        case 'uniform'
            idx = round(linspace(1, length(file_names), n));
        case 'random'
            idx = RI(n, length(file_names), false, true);
        otherwise
            error('unknown sampler');
    end
    file_names = file_names(idx);
end

imgs = map(@(f) get_img(f, sz), file_names);

function [img] = get_img(file_name, sz)

img = imread(file_name);
if ~isempty(sz) && any(sz(:) < [size(img,1); size(img,2)])
    sc = sz(:)./[size(img,1); size(img,2)];
    img = imresize(img, min(sc));
end
